/*Geektek Contacts is a program for storing contacts.
*Copyright (C) 2014  Aubrey Thomas
*
*GeekTek Contacts is free software: you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation, either version 3 of the License, or
*(at your option) any later version.
*
*GeekTek Contacts is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with GeekTek Contacts.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "addcontact.h"
#include "about.h"
#include "chooser.h"
#include "database.h"
#include "ui_addcontact.h"
#include "viewcontacts.h"
#include <QtSql/QtSql>
#include "QErrorMessage"
#include "QMessageBox"
#include <array>

/**
 * @brief AddContact::AddContact
 * @param parent
 */
AddContact::AddContact(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::AddContact)
{
    ui->setupUi(this);
    fromView = 0;
    db = new Database();
    connect(ui->addButton, SIGNAL(clicked()), this, SLOT(addContact()));
    connect(ui->cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
}

/**
 * @brief AddContact::addContact
 */
void AddContact::addContact() {
    std::array<QString,12> details = { ui->fName->text(), ui->lName->text(), ui->addr1->text(), ui->addr2->text(),
                            ui->addr3->text(), ui->county->text(), ui->postcode->text(), ui->country->text(), ui->email->text(),
                            ui->telephone->text(), ui->mobile->text(), ui->courses->text() };
    QSqlError query = db->addContact(details);
    if(query.number() == -1) {
        QMessageBox::information(this, "Success","Contact Added Successfully.");
    }
    else {
        QErrorMessage *err = new QErrorMessage();
        err->showMessage(query.text());
    }

}

/**
 * @brief AddContact::cancel
 */
void AddContact::cancel() {
    delete db;
    ///@todo: think about adding check.
    if(fromView) {
        window = new ViewContacts();
    }
    this->close();
    this->deleteLater();
    window->show();
}

/**
 * @brief AddContact::showAbout
 */
void AddContact::showAbout() {
    new About();
}

/**
 * @brief AddContact::~AddContact
 */
AddContact::~AddContact()
{
    delete ui;
}
