/*Geektek Contacts is a program for storing contacts.
*Copyright (C) 2014  Aubrey Thomas
*
*GeekTek Contacts is free software: you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation, either version 3 of the License, or
*(at your option) any later version.
*
*GeekTek Contacts is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with GeekTek Contacts.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "database.h"
#include <array>
#include <QtSql/QtSql>

Database::Database()
{
    if(!this->db.contains("Contacts")) {
        this->db = QSqlDatabase::addDatabase("QSQLITE", "Contacts");
    }
    else {
        this->db = QSqlDatabase::database("Contacts");
    }
    db.setDatabaseName(QDir::currentPath() + QDir::separator() + "contacts.db3");
    db.open();
    query = QSqlQuery(db);
}

/**
 * @brief Database::delContact
 * @param id
 * @return
 */
QSqlError Database::delContact(const int id)
{
    query.exec("SELECT id FROM contacts");
    query.seek(id);
    QString sql = QString("DELETE FROM contacts WHERE id=%1;").arg(query.value(0).toInt());
    query.exec(sql);
    return query.lastError();
}

/**
 * @brief Database::addContact
 * @param details
 * @return
 */
QSqlError Database::addContact(const std::array<QString,12> &details)
{
    query.prepare("INSERT INTO contacts(fname,lname,addr1,addr2,addr3,county,postcode,country,email,telephone,mobile,courses) VALUES (:fname,:lname,:addr1,:addr2,:addr3,:county,:pcode,:country,:email,:tphone,:mobile,"
                  ":courses)");
    query.bindValue(":fname", details[0]);
    query.bindValue(":lname", details[1]);
    query.bindValue(":addr1", details[2]);
    query.bindValue(":addr2", details[3]);
    query.bindValue(":addr3", details[4]);
    query.bindValue(":county", details[5]);
    query.bindValue(":pcode", details[6]);
    query.bindValue(":country", details[7]);
    query.bindValue(":email", details[8]);
    query.bindValue(":tphone", details[9]);
    query.bindValue(":mobile", details[10]);
    query.bindValue(":courses", details[11]);
    query.exec();
    return query.lastError();
}

/**
 * @brief Database::selectAllSeek
 * @param id
 * @return
 */
QSqlQuery Database::selectAllSeek(const int id)
{
    query.exec("SELECT * FROM contacts");
    query.seek(id);
    return query;
}

QSqlError Database::updateContact(const std::array<QString,12> &details, const int id)
{
    this->selectAllSeek(id);
    int uId = query.value(0).toInt();
    QString sql = QString("UPDATE contacts SET fname=:fname,lname=:lname,addr1=:addr1,addr2=:addr2,addr3=:addr3,county=:county,"
                          "postcode=:pcode, country=:country,email=:email,telephone=:tphone,mobile=:mobile,courses=:courses WHERE id=%1").arg(uId);
    query.prepare(sql);
    query.bindValue(":fname", details[0]);
    query.bindValue(":lname", details[1]);
    query.bindValue(":addr1", details[2]);
    query.bindValue(":addr2", details[3]);
    query.bindValue(":addr3", details[4]);
    query.bindValue(":county", details[5]);
    query.bindValue(":pcode", details[6]);
    query.bindValue(":country", details[7]);
    query.bindValue(":email", details[8]);
    query.bindValue(":tphone", details[9]);
    query.bindValue(":mobile", details[10]);
    query.bindValue(":courses", details[11]);
    query.exec();
    return query.lastError();
}

/**
 * @brief Database::create
 */
void Database::create()
{
    if(db.tables(QSql::Tables).count() == 0) {
        query.exec("CREATE TABLE config (version integer)");
        query.exec(QString("REPLACE INTO config (version) VALUES(%1)").arg(dbVersion));
        query.exec("CREATE TABLE IF NOT EXISTS contacts"
                   "(id integer primary key autoincrement,"
                   "fname varchar(20),"
                   "lname varchar(30),"
                   "addr1 varchar(20),"
                   "addr2 varchar(20),"
                   "addr3 varchar(20),"
                   "county varchar(30),"
                   "postcode varchar(10),"
                   "country varchar(50),"
                   "email varchar(30),"
                   "telephone varchar(10),"
                   "mobile varchar(10),"
                   "courses text)");
    }
    //check for database changes.
    else {
        query.exec("SELECT version FROM config");
        query.next();
        if(dbVersion != query.value(0).toInt()) {
            this->updateDB();
        }
    }
}

/**
 * @brief Database::updateDB
 */
void Database::updateDB()
{
    query.exec("CREATE TABLE IF NOT EXISTS config (version integer)");
    query.exec(QString("UPDATE config SET version=%1").arg(dbVersion));
    if(dbVersion == 2) {
        query.exec("ALTER TABLE contacts ADD COLUMN courses text");
    }
}
