/*Geektek Contacts is a program for storing contacts.
*Copyright (C) 2014  Aubrey Thomas
*
*GeekTek Contacts is free software: you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation, either version 3 of the License, or
*(at your option) any later version.
*
*GeekTek Contacts is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with GeekTek Contacts.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "about.h"
#include "database.h"
#include "editcontact.h"
#include "ui_editcontact.h"
#include "viewcontacts.h"
#include <QtSql/QtSql>
#include <QErrorMessage>
#include <QMessageBox>

/**
 * @brief EditContact::EditContact
 * @param parent
 */
EditContact::EditContact(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::EditContact)
{
    ui->setupUi(this);
    db = new Database();
    connect(ui->cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
    connect(ui->editButton, SIGNAL(clicked()), this, SLOT(edit()));
    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
}

/**
 * @brief EditContact::showEvent
 */
void EditContact::showEvent(QShowEvent *) {
    QSqlQuery query = db->selectAllSeek(this->id);
    ui->fName->setText(query.value(1).toString());
    ui->lName->setText(query.value(2).toString());
    ui->addr1->setText(query.value(3).toString());
    ui->addr2->setText(query.value(4).toString());
    ui->addr3->setText(query.value(5).toString());
    ui->county->setText(query.value(6).toString());
    ui->postcode->setText(query.value(7).toString());
    ui->country->setText(query.value(8).toString());
    ui->email->setText(query.value(9).toString());
    ui->telephone->setText(query.value(10).toString());
    ui->mobile->setText(query.value(11).toString());
    ui->courses->setText(query.value(12).toString());
    query.finish();
}

/**
 * @brief EditContact::cancel
 */
void EditContact::cancel() {
    delete db;
    ViewContacts *contacts = new ViewContacts();
    contacts->show();
    this->deleteLater();
    this->close();
}

/**
 * @brief EditContact::edit
 */
void EditContact::edit() {
    std::array<QString,12> details = { ui->fName->text(), ui->lName->text(), ui->addr1->text(), ui->addr2->text(),
                            ui->addr3->text(), ui->county->text(), ui->postcode->text(), ui->country->text(), ui->email->text(),
                            ui->telephone->text(), ui->mobile->text(), ui->courses->text() };
    QSqlError query = db->updateContact(details, this->id);
    if(query.number() == -1) {
        QMessageBox::information(this, "Success", "Contact Updated Successfully.");
    }
    else {
        QErrorMessage *err = new QErrorMessage();
        err->showMessage(query.text());
        err->adjustSize();
    }
}

/**
 * @brief EditContact::showAbout
 */
void EditContact::showAbout() {
    new About();
}

/**
 * @brief EditContact::~EditContact
 */
EditContact::~EditContact()
{
    delete ui;
}
