/*Geektek Contacts is a program for storing contacts.
*Copyright (C) 2014  Aubrey Thomas
*
*GeekTek Contacts is free software: you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation, either version 3 of the License, or
*(at your option) any later version.
*
*GeekTek Contacts is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with GeekTek Contacts.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "viewcontacts.h"
#include "ui_viewcontacts.h"
#include "addcontact.h"
#include "about.h"
#include "editcontact.h"
#include <QtSql/QtSql>
#include <QMessageBox>
#include <QErrorMessage>
#include <QtDebug>

/**
 * @brief ViewContacts::ViewContacts
 * @param parent
 */
ViewContacts::ViewContacts(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::ViewContacts)
{
    ui->setupUi(this);
    db = new Database();
    db->create();
    this->table();
    this->fields();
    connect(ui->addButton, SIGNAL(clicked()), this, SLOT(addContact()));
    connect(ui->deleteButton, SIGNAL(clicked()), this, SLOT(deleteContact()));
    connect(ui->editButton, SIGNAL(clicked()), this, SLOT(editContact()));
    connect(ui->search, SIGNAL(textChanged(QString)), this, SLOT(search()));
    connect(ui->fields, SIGNAL(currentIndexChanged(int)), this, SLOT(search()));
    connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
    connect(ui->tableView->horizontalHeader(), SIGNAL(sectionClicked(int)), this, SLOT(orderContacts(int)));
}

void ViewContacts::disableButtons()
{
    if(model->rowCount() == 0) {
        ui->editButton->setEnabled(0);
        ui->deleteButton->setEnabled(0);
    }
    else {
        ui->editButton->setEnabled(1);
        ui->deleteButton->setEnabled(1);
    }
}

void ViewContacts::fields()
{
    std::array<QString,12> fields = { "First Name","Last Name","Address 1","Address 2","Address 3","County",
                       "Postcode","Country","Email","Telephone","Mobile",
                       "Courses" };
    for(unsigned int i = 0 ; i < fields.size(); i++) {
        ui->fields->addItem(fields[i]);
    }
}

/**
 * @brief ViewContacts::table
 */
void ViewContacts::table()
{
    model = new QSqlQueryModel(this);
    query = QString("SELECT * FROM contacts");
    model->setQuery(query, this->db->db);
    model->removeColumn(0);
    model->setHeaderData(0, Qt::Horizontal, tr("First Name"));
    model->setHeaderData(1, Qt::Horizontal, tr("Last Name"));
    model->setHeaderData(2, Qt::Horizontal, tr("Address 1"));
    model->setHeaderData(3, Qt::Horizontal, tr("Address 2"));
    model->setHeaderData(4, Qt::Horizontal, tr("Address 3"));
    model->setHeaderData(5, Qt::Horizontal, tr("County"));
    model->setHeaderData(6, Qt::Horizontal, tr("Postcode"));
    model->setHeaderData(7, Qt::Horizontal, tr("Country"));
    model->setHeaderData(8, Qt::Horizontal, tr("Email Address"));
    model->setHeaderData(9, Qt::Horizontal, tr("Telephone"));
    model->setHeaderData(10, Qt::Horizontal, tr("Mobile"));
    model->setHeaderData(11, Qt::Horizontal, tr("Courses"));
    ui->tableView->setModel(model);
    this->disableButtons();
}

/**
 * @brief ViewContacts::addContact
 */
void ViewContacts::addContact()
{
    delete db;
    delete model;
    AddContact *add = new AddContact();
    this->close();
    add->fromView = 1;
    this->deleteLater();
    add->show();
}

/**
 * @brief ViewContacts::deleteContact
 */
void ViewContacts::deleteContact()
{
    int rId = ui->tableView->currentIndex().row();
    QSqlError query = db->delContact(rId);
    if(query.number() == -1) {
        this->table();
    }
    else {
        QErrorMessage *err = new QErrorMessage();
        err->showMessage(query.text());
    }
}

/**
 * @brief ViewContacts::editContact
 */
void ViewContacts::editContact()
{
    int rId = ui->tableView->currentIndex().row();
    delete model;
    delete db;
    EditContact *edit = new EditContact();
    edit->id = rId;
    this->close();
    this->deleteLater();
    edit->show();
}

/**
 * @brief ViewContacts::search
 */
void ViewContacts::search()
{
    std::array<QString,12> columns = { "fname","lname","addr1","addr2","addr3","county",
                       "postcode","country","email","telephone","mobile",
                       "courses" };
    query = QString("SELECT * FROM contacts WHERE %1 LIKE '\%%2\%'").arg(columns[ui->fields->currentIndex()],ui->search->text());
    model->setQuery(query, this->db->db);
    model->removeColumn(0);
    ui->tableView->setModel(model);
    ui->tableView->show();
    this->disableButtons();
}

/**
 * @brief ViewContacts::showAbout
 */
void ViewContacts::showAbout()
{
    new About();
}

void ViewContacts::orderContacts(const int column)
{
    std::array<QString,12> columns = { "fname","lname","addr1","addr2","addr3","county",
                       "postcode","country","email","telephone","mobile",
                       "courses" };
    unsigned int i = ui->tableView->horizontalHeader()->sortIndicatorSection();
    bool ascending = (i == (unsigned) column
    && ui->tableView->horizontalHeader()->sortIndicatorOrder() == Qt::DescendingOrder);
    Qt::SortOrder order = ascending ? Qt::DescendingOrder : Qt::AscendingOrder;
    QString asc = ascending ? "DESC" : "ASC";
    QString query = this->query + QString(" ORDER BY %1 %2").arg(columns[i], asc);
    model->setQuery(query, this->db->db);
    model->removeColumn(0);
    ui->tableView->setModel(model);
    ui->tableView->horizontalHeader()->setSortIndicator(column, order);
    ui->tableView->show();
}

/**
 * @brief ViewContacts::~ViewContacts
 */
ViewContacts::~ViewContacts()
{
    delete ui;
}
